

import java.util.Iterator;
import java.util.NoSuchElementException;

public class Tree<T extends Comparable<T>> implements Iterable<T> {
	private TreeNode<T> root; 
	private int size; 

	public Tree() {
		size = 0; 
	}

	// getters

	public int size() {
		return size; 
	}

	public boolean isEmpty() {
		return size == 0; 
	}

	public boolean contains(T val) {
		if(root == null) return false; 

		TreeNode<T> temp = search(root, val); 

		if(temp.getValue().equals(val)) return true; 
		return false; 

	}

	// if we don't find the value, return the parent so we know where to start again
	private TreeNode<T> search(TreeNode<T> node, T val) {
		if(node.getValue().equals(val)) return node; 

		if(node.getValue().compareTo(val) < 0) { 
			if(!node.hasRight()) return node; 
			return search(node.getRight(),val); 
		}

		if(!node.hasLeft()) return node; 

		return search(node.getLeft(), val); 

	}

	// ret false if value is already in the tree
	public boolean add(T val) { // O(nlogn)    // worst case: O(n^2) -- a self balancing tree will keep it at nlogn
		TreeNode<T> insert = new TreeNode<>(val); 

		if(size == 0) root = insert; 
		else { 
			TreeNode<T> temp = search(root,val); 
			if(temp.getValue().equals(val)) return false; 
			if(val.compareTo(temp.getValue()) < 0) {
				temp.setLeft(insert);
			} else {
				temp.setRight(insert);
			}
		}

		size++;
		return true; 
	}

	public boolean remove(T val) {
		TreeNode<T> temp = search(root,val); 

		if(temp.getValue().equals(val)) {
			temp.remove(); 
			return true; 
		}

		return false; 

	}



	public String toString() {
		return root.inOrder();  // inorder traversal of binary search tree will sort in ascending order 

	}

	private class BSTIterator implements Iterator {
		
		private TreeNode<T> cursor; 
		private TreeNode<T> recent; 

		public BSTIterator() {
			cursor = root.least(); 
		}

		@Override
		public boolean hasNext() {
			return cursor != null; 

		}

		@Override
		public T next() {
			if(cursor == null) throw new NoSuchElementException(); 
			recent = cursor; 
			cursor = cursor.successor(cursor); 
			return recent.getValue();
		}

	}
//
//	@Override
//	public boolean hasNext() {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public T next() {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
	@Override
	public Iterator<T> iterator() {
		// TODO Auto-generated method stub
		return new BSTIterator();
	}


}
